#include "../include/MetricsExporter.h"

#include <string>
#include <utility>
#include <vector>
#include <iostream>
#include <sstream>
#include <unistd.h>
#include <netinet/in.h>
#include <thread>
#include <chrono>
#include <cstring>
#include <random>
#include <ranges>
#include <sys/socket.h>

shared_ptr<unordered_map<uint32_t, Crawler>> crawler_;

void [[noreturn]] MetricsExporter::serve(shared_ptr<unordered_map<uint32_t, Crawler>> crawler) {
    crawler_ = move(crawler);

    // server_fd is a file descriptor.
    const int server_fd = socket(AF_INET, SOCK_STREAM, 0);

    sockaddr_in addr{};
    addr.sin_family = AF_INET;
    addr.sin_port = htons(8889);
    addr.sin_addr.s_addr = INADDR_ANY;

    if (bind(server_fd, reinterpret_cast<sockaddr *>(&addr), sizeof(addr)) != 0) {
        cout << "Please wait for the pipe to close.";
        return;
    }

    listen(server_fd, 50);

    cout << "Server is running on http://localhost:8889 \n";

    while (true) {
        int client_fd = accept(server_fd, nullptr, nullptr);

        thread(process_request, client_fd).detach();
    }

    close(server_fd);
}

void MetricsExporter::process_request(const int client_fd) {
    send_data(client_fd, HTML_RESPONSE_HEADER_lol);

    ostringstream oss;

    oss << BEGINNING;

    for (auto it = crawler_->begin(); it != crawler_->end(); ++it) {
        oss << it->second.links_pressed << ": " << it->second.user_agent << endl;
    }

    oss << END;

    send_data(client_fd, oss.str());
    send(client_fd, "0\r\n\r\n", 5, 0);
    close(client_fd);
}

void MetricsExporter::send_data(const int client_fd, const string& data) {
    ostringstream oss;

    oss << hex << data.size() << "\r\n" << data << "\r\n";

    send(client_fd, oss.str().c_str(), oss.str().size(), 0);
}